from flask import request
from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal import logger
from portal.rest import rest_helpers
from portal.rest.schemas import SCHEMAS
from portal.service import general, permissions
from portal.service.groups import GroupService

groups_namespace = Namespace('')

log = logger.get_logger(__name__)


@groups_namespace.route('/courses/<string:cid>/groups')
@groups_namespace.param('cid', 'Course id')
@groups_namespace.response(404, 'Course not found')
class GroupsList(Resource):
    @jwt_required
    # @groups_namespace.response(200, 'List of all groups', model=groups_schema)
    def get(self, cid: str):
        course = general.find_course(cid)
        groups = GroupService().list_groups(course)
        return SCHEMAS.dump('groups', groups)

    @jwt_required
    # @groups_namespace.response(201, 'Group created', model=group_schema)
    @groups_namespace.response(403, 'Not allowed to create group')
    def post(self, cid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.update_course()

        data = rest_helpers.parse_request_data(action='create', resource='group')

        new_group = GroupService().create_group(course, **data)
        return SCHEMAS.dump('group', new_group), 201


@groups_namespace.route('/courses/<string:cid>/groups/<string:gid>')
@groups_namespace.param('cid', 'Course id')
@groups_namespace.param('gid', 'Group id')
@groups_namespace.response(404, 'Course not found')
@groups_namespace.response(404, 'Group not found')
class GroupResource(Resource):
    @jwt_required
    # @groups_namespace.response(201, 'Group', model=group_schema)
    @groups_namespace.response(403, 'Not allowed to get group')
    def get(self, cid: str, gid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.view_course()

        group = general.find_group(course, gid)
        return SCHEMAS.dump('group', group)

    @jwt_required
    @groups_namespace.response(204, 'Group deleted')
    @groups_namespace.response(403, 'Not allowed to delete group')
    def delete(self, cid: str, gid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.update_course()

        group = general.find_group(course, gid)
        GroupService(group).delete_group()
        return '', 204

    @jwt_required
    @groups_namespace.response(204, 'Group updated')
    @groups_namespace.response(403, 'Not allowed to update group')
    def put(self, cid: str, gid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.write_groups()

        data = rest_helpers.parse_request_data(action='update', resource='group', partial=True)
        group = general.find_group(course, gid)
        GroupService(group).update_group(data)
        return '', 204


@groups_namespace.route('/courses/<string:cid>/groups/<string:gid>/users')
@groups_namespace.param('cid', 'Course id')
@groups_namespace.param('gid', 'Group id')
class GroupUsersList(Resource):
    @jwt_required
    # @groups_namespace.response(200, 'List of users in the group', model=users_schema)
    @groups_namespace.response(403, 'Not allowed to get users in the group')
    def get(self, cid: str, gid: str):
        course = general.find_course(cid)
        role_id = request.args.get('role')

        group = general.find_group(course, gid)
        # authorization
        permissions.PermissionsService(course=course).require.belongs_to_group(group)

        users = GroupService(group).find_users_by_role(course, role_id)
        return SCHEMAS.dump('users', users)

    @jwt_required
    @groups_namespace.response(204, 'User\'s list updated')
    @groups_namespace.response(403, 'Not allowed to add users to the group')
    def put(self, cid: str, gid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.write_groups()

        data = rest_helpers.parse_request_data(
            schema=SCHEMAS.client_list_update, action='update', resource='group_membership'
        )

        group = general.find_group(course, gid)
        # everything from users_add is added, THEN everything from users_remove
        # is removed
        GroupService(group).update_membership(data)
        return '', 204


@groups_namespace.route(
    '/courses/<string:cid>/groups/<string:gid>/users/<string:uid>')
@groups_namespace.param('cid', 'Course id')
@groups_namespace.param('gid', 'Group id')
@groups_namespace.param('uid', 'User id')
@groups_namespace.response(404, 'Course not found')
@groups_namespace.response(404, 'Group not found')
@groups_namespace.response(404, 'User not found')
class GroupAddOrDeleteSingleUser(Resource):
    @jwt_required
    @groups_namespace.response(204, 'User\'s list updated')
    @groups_namespace.response(403, 'Not allowed to add user to the group')
    def put(self, cid: str, gid: str, uid: str):
        course = general.find_course(cid)

        # authorization
        permissions.PermissionsService(course=course).require.write_groups()

        group = general.find_group(course, gid)
        user = general.find_user(uid)
        GroupService(group).add_user(user)
        return '', 204

    @jwt_required
    @groups_namespace.response(204, 'User\'s list updated')
    @groups_namespace.response(403, 'Not allowed to delete users to the group')
    def delete(self, cid: str, gid: str, uid: str):
        course = general.find_course(cid)

        permissions.PermissionsService(course=course).require.write_groups()

        group = general.find_group(course, gid)
        user = general.find_user(uid)
        GroupService(group).remove_user(user)
        return '', 204


@groups_namespace.route('/courses/<string:cid>/groups/<string:gid>/projects')
@groups_namespace.param('cid', 'Course id')
@groups_namespace.param('gid', 'Group id')
@groups_namespace.response(404, 'Course not found')
@groups_namespace.response(404, 'Group not found')
class GroupProjectsList(Resource):
    @jwt_required
    def get(self, cid: str, gid: str):
        course = general.find_course(cid)
        group = general.find_group(course, gid)
        # authorization
        permissions.PermissionsService(course=course).require.belongs_to_group(group)

        projects = GroupService(group).find_projects()
        return SCHEMAS.dump('projects', projects)


@groups_namespace.route(
    '/courses/<string:cid>/groups/<string:gid>/projects/<string:pid>')
@groups_namespace.param('cid', 'Course id')
@groups_namespace.param('gid', 'Group id')
@groups_namespace.param('pid', 'Project id')
@groups_namespace.response(404, 'Course not found')
@groups_namespace.response(404, 'Group not found')
@groups_namespace.response(404, 'Project not found')
class GroupAddOrDeleteProject(Resource):
    @jwt_required
    @groups_namespace.response(204, 'Projects\'s list updated')
    @groups_namespace.response(403, 'Not allowed to add project to the group')
    def put(self, cid: str, gid: str, pid: str):
        course = general.find_course(cid)

        # authorization
        permissions.PermissionsService(course=course).require.write_groups()

        group = general.find_group(course, gid)
        project = general.find_project(course, pid)
        GroupService(group).add_project(project)
        return '', 204

    @jwt_required
    @groups_namespace.response(204, 'Projects\'s list updated')
    @groups_namespace.response(
        403, 'Not allowed to delete project from the group')
    def delete(self, cid: str, gid: str, pid: str):
        course = general.find_course(cid)

        # authorization
        permissions.PermissionsService(course=course).require.write_groups()

        group = general.find_group(course, gid)
        project = general.find_project(course, pid)
        GroupService(group).remove_project(project)
        return '', 204


@groups_namespace.route('/courses/<string:cid>/groups/import')
@groups_namespace.param('cid', 'Course id')
@groups_namespace.response(404, 'Course not found')
class GroupImport(Resource):
    @jwt_required
    @groups_namespace.response(201, 'Group import')
    @groups_namespace.response(404, 'Group not found')
    @groups_namespace.response(403, 'Not allowed to import to the group')
    def put(self, cid: str):
        target_course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=target_course).require.write_groups()
        data = rest_helpers.parse_request_data(
            schema=SCHEMAS.group_import, action='import', resource='group'
        )
        new_group = GroupService().import_group(data, target_course)
        return SCHEMAS.dump('group', new_group), 201
