from flask import request
from flask_jwt_extended import create_access_token, create_refresh_token, get_jwt_identity, \
    jwt_refresh_token_required, jwt_required
from flask_restplus import Namespace, Resource, fields

from portal import logger
from portal.database.models import Client
from portal.service.auth import login_gitlab, login_secret, login_username_password
from portal.service.errors import PortalAPIError, UnauthorizedError

log = logger.get_logger(__name__)

auth_namespace = Namespace('auth')

login_schema = auth_namespace.model('LoginSchema', {
    'id': fields.String,
    'access_token': fields.String,
    'refresh_token': fields.String
})

refresh_schema = auth_namespace.model('RefreshSchema', {
    'access_token': fields.String
})

logout_schema = auth_namespace.model('LogoutSchema', {
    'access_token': fields.String,
    'refresh_token': fields.String
})


@auth_namespace.route('/login')
class Login(Resource):
    @auth_namespace.marshal_with(login_schema)
    @auth_namespace.response(401, 'Client is not authorized')
    @auth_namespace.response(400, 'Missing login type.')
    @auth_namespace.response(400, 'Missing component name.')
    @auth_namespace.response(400, 'Missing component secret.')
    @auth_namespace.response(400, 'Invalid login type.')
    def post(self):
        data = request.get_json()
        client = _login_client(data)
        return dict(id=client.id,
                    access_token=create_access_token(identity=client.id),
                    refresh_token=create_refresh_token(identity=client.id))


@auth_namespace.route('/refresh')
class Refresh(Resource):
    @jwt_refresh_token_required
    @auth_namespace.marshal_with(refresh_schema)
    @auth_namespace.response(401, 'Client is not authorized')
    def post(self):
        client_id = authorized_client()
        return dict(access_token=create_access_token(identity=client_id))


@auth_namespace.route('/logout')
class Logout(Resource):
    @jwt_required
    @auth_namespace.marshal_with(logout_schema)
    @auth_namespace.response(401, 'Client is not authorized')
    def post(self):
        client_id = authorized_client()
        return dict(id=client_id, access_token=None, refresh_token=None)


def authorized_client() -> str:
    client = get_jwt_identity()
    if not client:
        raise UnauthorizedError()
    return client


def _login_client(data: dict) -> Client:
    if not data.get('type'):
        raise PortalAPIError(400, message="Missing login type.")

    login_type = data['type']
    types = dict(gitlab=login_gitlab,
                 username_password=login_username_password,
                 secret=login_secret)

    if login_type not in types.keys():
        raise PortalAPIError(400, message="Invalid login type.")
    identifier = data.get('identifier', None)
    secret = data.get('secret', None)

    return types[login_type](identifier, secret)
