from flask import request
from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal import logger
from portal.database.models import ClientType
from portal.rest import rest_helpers
from portal.rest.schemas import SCHEMAS
from portal.service import general, permissions
from portal.service.roles import RoleService

roles_namespace = Namespace('')

log = logger.get_logger(__name__)


@roles_namespace.route('/courses/<string:cid>/roles')
@roles_namespace.param('cid', 'Course id')
class RoleList(Resource):
    @jwt_required
    # @roles_namespace.response(200, 'Roles list', model=roles_schema)
    def get(self, cid):
        course = general.find_course(cid)
        roles = RoleService().list_roles(course)
        return SCHEMAS.dump('roles', roles)

    @jwt_required
    # @roles_namespace.response(201, 'Role created', model=role_schema)
    def post(self, cid):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.update_course()

        data = rest_helpers.parse_request_data(action='create', resource='role')
        new_role = RoleService().create_role(course, **data)
        return SCHEMAS.dump('role', new_role), 201


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>')
@roles_namespace.param('cid', 'Course id')
@roles_namespace.param('rid', 'Role id')
@roles_namespace.response(404, 'Course not found')
@roles_namespace.response(404, 'Role not found')
class RoleResource(Resource):
    @jwt_required
    # @roles_namespace.response(200, 'Role in the course', model=role_schema)
    def get(self, cid: str, rid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.view_course()

        role = general.find_role(course, rid)
        return SCHEMAS.dump('role', role)

    @jwt_required
    @roles_namespace.response(204, 'Deleted role')
    def delete(self, cid, rid):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.update_course()

        role = general.find_role(course, rid)
        RoleService(role).delete_role()
        return '', 204

    @jwt_required
    @roles_namespace.response(204, 'Role updated')
    def put(self, cid: str, rid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.write_roles()

        data = rest_helpers.parse_request_data(action='update', resource='role', partial=True)

        role = general.find_role(course, rid)
        RoleService(role).update_role(data)
        return '', 204


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>/permissions')
@roles_namespace.param('cid', 'Course id')
@roles_namespace.param('rid', 'Role id')
@roles_namespace.response(404, 'Course not found')
@roles_namespace.response(404, 'Role not found')
class RolePermissions(Resource):
    @jwt_required
    # @roles_namespace.response(200, 'Get permissions for role', model=permissions_schema)
    def get(self, cid: str, rid: str):
        course = general.find_course(cid)
        role = general.find_role(course, rid)

        # authorization
        permissions.PermissionsService(course=course).require.belongs_to_role(role)

        return SCHEMAS.dump('permissions', role.permissions)

    @jwt_required
    # @roles_namespace.response(204, 'Update permissions for course', model=permissions_schema)
    def put(self, cid: str, rid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.write_roles()

        data = rest_helpers.parse_request_data(action='update', resource='permissions')

        role = general.find_role(course, rid)
        RoleService(role).update_permissions(data)
        return SCHEMAS.dump('permissions', role.permissions), 200


@roles_namespace.route('/courses/<string:cid>/roles/<string:rid>/clients')
@roles_namespace.param('cid', 'Course id')
@roles_namespace.param('rid', 'Role id')
@roles_namespace.response(404, 'Course not found')
@roles_namespace.response(404, 'Role not found')
class RoleUsersList(Resource):
    @jwt_required
    # @roles_namespace.response(200, 'Gets users in the role', model=users_schema)
    def get(self, cid: str, rid: str):
        course = general.find_course(cid)
        role = general.find_role(course, rid)
        type = request.args.get('type')
        # authorization

        permissions.PermissionsService(course=course).require.belongs_to_role(role)

        users = [client for client in role.clients if client.type == ClientType[type.upper()]]
        return SCHEMAS.dump('users', users)

    @jwt_required
    @roles_namespace.response(204, 'Updates users membership in role')
    def put(self, cid: str, rid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.write_roles()

        data = rest_helpers.parse_request_data(
            schema=SCHEMAS.client_list_update, action='update', resource='role_membership'
        )

        # everything from users_add is added, THEN everything from users_remove
        # is subtracted
        role = general.find_role(course, rid)
        RoleService(role).update_clients_membership(data)
        return '', 204


@roles_namespace.route(
    '/courses/<string:cid>/roles/<string:rid>/clients/<string:clid>')
@roles_namespace.param('cid', 'Course id')
@roles_namespace.param('rid', 'Role id')
@roles_namespace.param('clid', 'Client id')
@roles_namespace.response(404, 'Course not found')
@roles_namespace.response(404, 'Role not found')
@roles_namespace.response(404, 'Client not found')
class RoleClient(Resource):
    @jwt_required
    @roles_namespace.response(204, 'Adds permissions to role')
    def put(self, cid: str, rid: str, clid: str):
        course = general.find_course(cid)
        # authorization
        permissions.PermissionsService(course=course).require.write_roles()
        role = general.find_role(course, rid)
        client_type = request.args.get('type')
        client = general.find_client(clid, client_type=client_type)
        RoleService(role).add_client(client)
        return '', 204

    @jwt_required
    @roles_namespace.response(204, 'Removes permissions from role')
    def delete(self, cid: str, rid: str, clid: str):
        course = general.find_course(cid)
        permissions.PermissionsService(course=course).require.write_roles()
        role = general.find_role(course, rid)
        client_type = request.args.get('type')
        client = general.find_client(clid, client_type=client_type)
        RoleService(role).remove_client(client)
        return '', 204
