"""
Users REST API module
"""

from flask import request
from flask_jwt_extended import jwt_required
from flask_restplus import Namespace, Resource

from portal import logger
from portal.database.models import User
from portal.rest import rest_helpers
from portal.rest.schemas import SCHEMAS
from portal.service import auth, errors, general, permissions
from portal.service.general import find_client_owner
from portal.service.secrets import create_secret, delete_secret
from portal.service.users import UserService

users_namespace = Namespace('users')

log = logger.get_logger(__name__)


@users_namespace.route('')
class UserList(Resource):
    @jwt_required
    # @users_namespace.response(200, 'List of users', model=users_schema)
    def get(self):
        permissions.PermissionsService().require.sysadmin()
        course_id = request.args.get('course')
        group_id = request.args.get('group')
        if group_id and not course_id:
            raise errors.PortalAPIError(400,
                                        "Invalid filter combination at /get/users: "
                                        "missing course id.")

        UserService()
        filtered_users = UserService().find_users_filtered(
            course_id=course_id, group_id=group_id)
        return SCHEMAS.dump('users', filtered_users), 200

    @jwt_required
    # @users_namespace.response(201, 'Created user', model=user_schema)
    def post(self):
        permissions.PermissionsService().require.sysadmin()
        data = rest_helpers.parse_request_data(action='create', resource='user')
        new_user = UserService().create_user(**data)
        return SCHEMAS.dump('user', new_user), 201


@users_namespace.route('/<string:uid>')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserResource(Resource):
    @jwt_required
    # @users_namespace.response(200, "User reduced info", model=user_schema_reduced)
    # @users_namespace.response(200, "User info", model=users_schema)
    def get(self, uid: str):
        user = general.find_user(uid)
        # authorization
        schema = get_user_schema_based_on_permissions(user)
        return schema.dump(user)[0], 200

    @jwt_required
    @users_namespace.response(204, 'User updated')
    def put(self, uid: str):
        user = general.find_user(uid)
        permissions_service = permissions.PermissionsService()
        permissions_service.require.sysadmin_or_self(uid)

        data = rest_helpers.parse_request_data(action='update', resource='user', partial=True)
        UserService(user=user).update_user(data)
        return '', 204

    @jwt_required
    @users_namespace.response(204, 'User deleted')
    def delete(self, uid: str):
        permissions.PermissionsService().require.sysadmin()
        user = general.find_user(uid)
        UserService(user=user).delete_user()
        return '', 204


@users_namespace.route('/<string:uid>/password')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserPassword(Resource):
    @jwt_required
    @users_namespace.response(204, 'User password updated')
    def put(self, uid: str):
        user = general.find_user(uid)
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        data = rest_helpers.parse_request_data(action='update', resource='password_change')
        UserService(user=user).update_password(data)
        return '', 204


@users_namespace.route('/<string:uid>/submissions')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserSubmissionList(Resource):
    @jwt_required
    # @users_namespace.response(200, 'Submissions list', model=submissions_schema)
    def get(self, uid: str):
        client = auth.find_client()
        user = general.find_user(uid)

        course_id = request.args.get('course')
        project_ids = request.args.getlist('project')
        if project_ids and not course_id:
            raise errors.PortalAPIError(400,
                                        f"Invalid filter combination at "
                                        f"/get/users/{uid}/submissions: missing course id.")

        submissions = get_submissions_based_on_permissions(
            client, user, course_id, project_ids)
        return SCHEMAS.dump('submissions', submissions), 200


def find_groups_intersection(user1: User, user2: User):
    other_groups = user2.groups
    res = [group for group in user1.groups if group in other_groups]
    return res


@users_namespace.route('/<string:uid>/roles')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserRoleList(Resource):
    @jwt_required
    # @users_namespace.response(200, 'User roles list', model=roles_schema)
    def get(self, uid: str):
        user = general.find_user(uid)
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        course_id = request.args.get('course')
        roles = UserService(user).find_roles_filtered(course_id=course_id)
        return SCHEMAS.dump('roles', roles)


@users_namespace.route('/<string:uid>/courses')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserCourseList(Resource):
    @jwt_required
    # @users_namespace.response(200, 'Courses roles list', model=courses_schema)
    def get(self, uid: str):
        user = general.find_user(uid)
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        return SCHEMAS.dump('courses', user.courses)


@users_namespace.route('/<string:uid>/projects')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserProjectsList(Resource):
    @jwt_required
    # @users_namespace.response(200, 'Courses roles list', model=courses_schema)
    def get(self, uid: str):
        user = general.find_user(uid)
        # authorization
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        projects = UserService(user=user).find_all_projects_for_user()
        return SCHEMAS.dump('projects', projects)


@users_namespace.route('/<string:uid>/groups')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserGroupList(Resource):
    @jwt_required
    # @users_namespace.response(200, 'User groups list', model=groups_schema)
    def get(self, uid: str):
        user = general.find_user(uid)
        # authorization
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        course_id = request.args.get('course')
        groups = UserService(user=user).find_groups_filtered(course_id=course_id)
        return SCHEMAS.dump('groups', groups)


@users_namespace.route('/<string:uid>/reviews')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserReviewList(Resource):
    @jwt_required
    # @users_namespace.response(200, 'User reviews list', model=reviews_schema)
    def get(self, uid: str):
        user = general.find_user(uid)
        # authorization
        permissions.PermissionsService().require.sysadmin_or_self(uid)

        reviews = UserService(user=user).find_reviews()
        return SCHEMAS.dump('reviews', reviews)


@users_namespace.route('/<string:uid>/permissions')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserEffectivePermissions(Resource):
    @jwt_required
    @users_namespace.response(200, 'Effective permissions')
    def get(self, uid: str):
        user = general.find_user(uid)
        # authorization
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        course_id = request.args.get('course')
        perm = permissions.PermissionsService(client=user).get_effective_permissions(course_id)
        return perm


@users_namespace.route('/<string:uid>/secrets')
@users_namespace.param('uid', "User's id")
@users_namespace.response(404, 'User not found')
class UserSecretsController(Resource):
    @jwt_required
    @users_namespace.response(200, 'List user secrets')
    def get(self, uid: str):
        user = general.find_user(uid)
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        return SCHEMAS.dump('secrets', user.secrets)

    @jwt_required
    # @users_namespace.response(201, 'Created user secret', model=secret_schema)
    def post(self, uid):
        user = general.find_user(uid)
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        data = rest_helpers.parse_request_data(action='create', resource='secret')
        new_secret, value = create_secret(user, **data)
        return {'id': new_secret.id, 'value': value}, 201


@users_namespace.route('/<string:uid>/secrets/<string:sid>')
@users_namespace.param('uid', "User's id")
@users_namespace.param('sid', "Secret id")
@users_namespace.response(404, 'User not found')
class UserSecretController(Resource):
    @jwt_required
    @users_namespace.response(200, 'User secret detail')
    def get(self, uid: str, sid: str):
        permissions.PermissionsService().require.sysadmin()
        user = general.find_user(uid)
        secret = general.find_secret(user, sid)
        return SCHEMAS.dump('secret', secret)

    @jwt_required
    @users_namespace.response(204, 'User secret deleted')
    def delete(self, uid: str, sid: str):
        user = general.find_user(uid)
        permissions.PermissionsService().require.sysadmin_or_self(uid)
        delete_secret(user, sid)
        return '', 204

    @jwt_required
    @users_namespace.response(204, 'User secret updated')
    @users_namespace.response(403, 'Not allowed to update user secret')
    def put(self, uid: str, sid: str):
        permissions.PermissionsService().require.sysadmin_or_self(uid)

        data = rest_helpers.parse_request_data(action='update', resource='secret', partial=True)

        user = general.find_user(uid)
        secret = general.find_secret(user, sid)
        general.update_entity(secret, data, ['name', 'expires_at'])
        return '', 204


#
# Helper functions
#


def get_user_schema_based_on_permissions(user):
    if permissions.PermissionsService().check.sysadmin_or_self(user.id):
        return SCHEMAS.user()
    return SCHEMAS.user_reduced()


def get_submissions_based_on_permissions(client, user, course_id, project_ids):
    perm_service = permissions.PermissionsService()
    if course_id:
        return get_submissions_based_on_permissions_for_course(
            client, course_id, project_ids, user)
    if perm_service.check.sysadmin():
        return user.submissions
    if find_client_owner(client) == user:
        return user.submissions
    else:
        raise errors.ForbiddenError(client)


def get_submissions_based_on_permissions_for_course(client, course_id, project_ids, user):
    course = general.find_course(course_id)
    client_owner = find_client_owner(client)
    perm_service = permissions.PermissionsService(course=course)
    checks = [
        perm_service.check.sysadmin(),
        perm_service.check.permissions(['read_submissions_all']),
        (perm_service.check.permissions(['read_submissions_groups'])
         and find_groups_intersection(client_owner, user)),
        (perm_service.check.permissions(['read_submissions_own'])
         and client_owner == user)
    ]
    perm_service.require.any_check(client, *checks)

    return UserService(user=user).find_submissions_filtered(
        course_id=course_id, project_ids=project_ids)
