"""
Authentication service
"""
import logging

from flask_jwt_extended import get_jwt_identity

from portal import gitlab_factory
from portal.database.models import Client
from portal.service import general, errors

log = logging.getLogger(__name__)


def login_gitlab(identifier: str, secret: str) -> Client:
    """Verifies GitLab access token and username combination.

        Args:
            identifier(str): username of the user attempting to log in
            secret(str): access token string from gitlab

        Returns(User): the authenticated user
        """
    if secret is None:
        raise errors.PortalAPIError(400, 'No gitlab access token found.')

    validate_gitlab_token(secret, username=identifier)

    user = general.find_user(identifier, throws=False)
    if user is None:
        raise errors.InvalidGitlabAccessTokenError()
    return user


def login_username_password(identifier: str, secret: str) -> Client:
    """Verifies user login credentials.
    Args:
        identifier(str): username to log in
        secret(str): user's password

    Returns(Client): the authenticated user

    """
    user = general.find_user(identifier, throws=False)
    if user is None or secret is None:
        raise errors.IncorrectCredentialsError()

    if user.verify_password(password=secret):
        return user

    raise errors.IncorrectCredentialsError()


def login_secret(identifier: str, secret: str) -> Client:
    """Verifies the provided Client secret.

    Args:
        identifier: identifier of the Client to log in
        secret: the provided secret

    Returns:

    """
    client = __find_client_helper(identifier)
    if client.verify_secret(secret):
        return client
    raise errors.UnauthorizedError(f"[LOGIN] Invalid secret.")


def validate_gitlab_token(token: str, username: str, throws: bool = True):
    """Validates gitlab access token using the gitlab permissions
    Args:
        token(str): Gitlab access token
        username(str): Username
        throws(bool): Throws an exception if the token is not valid
    Returns(Bool):
    """
    client = gitlab_factory.instance(oauth_token=token)
    client.auth()
    user = client.user
    if user.username != username:
        if throws:
            raise errors.InvalidGitlabAccessTokenError()
        return False
    return True


def find_client() -> Client:
    identifier = get_jwt_identity()
    return __find_client_helper(identifier)


def __find_client_helper(identifier: str) -> Client:
    log.debug(f"[LOGIN] Finding client using identifier: {identifier}")
    client = general.find_client(identifier, throws=False)
    if not client:
        raise errors.UnauthorizedError(f"[LOGIN] Unknown client identifier {identifier}.")
    return client
