"""
Courses service
"""

from typing import List

from portal import logger
from portal.database.models import Course, Group, Role, User
from portal.service import general
from portal.service.groups import GroupService
from portal.service.projects import ProjectService
from portal.service.roles import RoleService

log = logger.get_logger(__name__)


def _set_course_props(course: Course, data: dict):
    return general.update_entity(course, data, allowed=['name', 'codename', 'description'])


class CourseService:
    def __init__(self, course: Course = None):
        self._course = course

    @property
    def course(self):
        return self._course

    def create_course(self, **data) -> Course:
        """Creates new course

        Keyword Args:
            name(str): name of the course
            codename(str): codename of the course

        Returns(Course): Course instance
        """
        course = Course()
        self._course = course
        _set_course_props(course=course, data=data)
        log.debug(f"[CREATE] Course {course.id}: {course}")
        return course

    def copy_course(self, target: Course, config: dict) -> Course:
        """Copies course and it's other resources (roles, groups, projects)
        The other resources that should be copied are specified in the config

        Args:
            target(Course): To which course the resources will be copied
            config(dict): Configuration to specify which resource should be copied

        Returns(Course): Copied course instance (target)
        """
        if config.get('roles'):
            for role in self.course.roles:
                RoleService(role).copy_role(target, with_clients=config['roles'])
        if config.get('groups'):
            for group in self.course.groups:
                GroupService(group).copy_group(target, with_users=config['groups'])
        if config.get('projects'):
            for project in self.course.projects:
                ProjectService(project).copy_project(target)

        general.write_entity(target)
        log.info(f"[IMPORT] From {self.course.id} to:  {target.id}.")

        return target

    def delete_course(self):
        """Deletes course
        """
        log.info(f"[DELETE] Course: {self.course.id} ({self.course.codename})")
        general.delete_entity(self.course)

    def update_course(self, data: dict) -> Course:
        """Updates course

        Args:
            data(dict): Dictionary containing data which should be changed

        Returns(Course): Updated course instance
        """
        _set_course_props(course=self.course, data=data)
        log.info(f"[UPDATE] Course {self.course.id} ({self.course.codename}): {self.course}.")
        return self.course

    def update_notes_token(self, token: str) -> Course:
        """Updates notes access token of a course.

        Args:
            token(str): The new token

        Returns(Course): Course instance
        """
        self.course.notes_access_token = token
        general.write_entity(self.course)
        log.info(f"[UPDATE] Notes access token "
                 f"({self.course.codename}) [{self.course.id}]: {token}")
        return self.course

    def find_all_courses(self) -> List[Course]:
        """Find all courses
        Returns(list): List of courses
        """
        return Course.query.all()

    def get_users_filtered(self, groups: List[str], roles: List[str]) -> List[User]:
        """Get all users for course filtered
        Args:
            groups(list): Group names list
            roles(list):  Role names list
        Returns(List[User]):
        """

        groups_entities = Group.query.filter(Group.id.in_(groups)).all()
        roles_entities = Role.query.filter(Role.id.in_(roles)).all()
        return self.course.get_users_filtered(groups_entities, roles_entities)
