"""
LDAP integration module
"""
from flask import Flask
import ldap
from ldap.ldapobject import LDAPObject
import logging

from portal import logger

log = logger.get_logger(__name__)


class LDAPWrapper(object):
    def __init__(self, app: Flask = None):
        self.flask_app = None
        self._ldap_client = None
        self._selector_base = ',dc=fi,dc=muni,dc=cz'
        if app:
            self.init_app(app)

    def init_app(self, app: Flask):
        self.flask_app = app
        self._ldap_client = None
        return app

    @property
    def ldap_url(self) -> str:
        """Returns the LDAP server URL
        Returns(str): URL to the LDAP server
        """
        return self.flask_app.config.get('LDAP_URL', None)

    @property
    def is_enabled(self) -> bool:
        """Whether the LDAP extension is enabled

        LDAP is enabled when LDAP url is set
        Returns(bool): true if is enabled, otherwise false
        """
        return self.ldap_url is not None

    @property
    def selector_base(self) -> str:
        """Base selector, for FI muni it is: 'dc=fi,dc=muni,dc=cz'

        Returns(str): Selector base string
        """
        return self._selector_base

    @property
    def ldap(self) -> LDAPObject:
        """LDAP object instance
        Returns(LDAPObject): LDAP Object instance
        """
        if not self._ldap_client and self.is_enabled:
            self._ldap_client = ldap.initialize(self.ldap_url, bytes_mode=False)
        return self._ldap_client

    def search(self, selector: str):
        selector = selector + self.selector_base
        result = self.ldap.search_s(selector, ldap.SCOPE_SUBTREE, "objectclass=*")
        log.debug(f"[LDAP] Search ({selector}): {result}")
        return result

    def search_dict(self, *args, **kwargs):
        l_res = self.search(*args, **kwargs)
        if not l_res:
            return None
        return [rec[1] for rec in l_res]
